﻿import re
import csv
import sys

class TorqueConfigEntry:
   def __init__(self, line, keyPrefix, key, index, value):
      self.line      = line
      self.keyPrefix = keyPrefix
      self.key       = key
      self.index     = index
      self.value     = value
      
def sortKey(entry):
   if not entry.keyPrefix or not entry.key or not entry.index:
      return '0_' + entry.line
   else:
      return '1_' + entry.keyPrefix + '_{:03}'.format(int(entry.index)) + '_' + entry.key

if len(sys.argv) < 2:
   print('Usage: {} <torqueConf.dat>', sys.argv[0])
   sys.exit(-1)
      
inputFile = sys.argv[1]

with open(inputFile, 'r') as input:
   categories = dict()
   for line in input:
      match = re.match('(log_|obd_|)([^\d]+)(\d*)=(.*)', line)
      (keyPrefix, key, index, value) = match.groups() if match is not None else ('', '', '', '')
      if not index:
         (keyPrefix, key, index, value) = ('', '', '', '')
      if keyPrefix not in categories:
         categories[keyPrefix] = []
      categories[keyPrefix] += [TorqueConfigEntry(line.rstrip(), keyPrefix, key, index, value)]

for (keyPrefix, logEntries) in categories.items():
   outputFile = inputFile + '_' + (keyPrefix[:-1] if keyPrefix else 'general') + '.csv'
   print(outputFile)
   with open(outputFile, 'w') as output:
      fieldnames=['Name', 'ShortName', 'ModeAndPID', 'Equation', 'Min Value', 'Max Value', 'Units', 'Header', 'scale', 'userpid', 'varType', 'class', 'mode', 'line']
      writer = csv.DictWriter(output, fieldnames=fieldnames, delimiter=',', lineterminator='\n') #, quoting=csv.QUOTE_ALL)
      writer.writeheader()
      content = dict()
      index = None
      mapping = {'equation'  : 'Equation',
                 'fullName'  : 'Name',
                 'header'    : 'Header',
                 'max'       : 'Max Value',
                 'min'       : 'Min Value',
                 'pid'       : 'ModeAndPID',
                 'scale'     : 'scale',
                 'shortName' : 'ShortName',
                 'unit'      : 'Units',
                 'userpid'   : 'userpid',
                 'varType'   : 'varType',
                 'class'     : 'class',
                 'mode'      : 'mode'}
      for entry in sorted(logEntries, key=sortKey):
         if index is not None and entry.index != index and len(content) > 0:
            writer.writerow(content)
            content.clear()
         index = entry.index
         if entry.key and entry.key in mapping:
            if entry.key == 'pid':
               content[mapping[entry.key]] = '{:X}'.format(int(entry.value))
            else:
               content[mapping[entry.key]] = entry.value
         else:
            content['line'] = entry.line
            writer.writerow(content)
            content.clear()
      if len(content) > 0:
         writer.writerow(content)
   